# Compile source code, examples and utils common for all targets.
# Don't use this makefile directly, it is included by the other *.mak's!
# If you have problems using make.exe change the label names bellow.



SRC_DIR  = src/

# components source
COM_DIR  = $(SRC_DIR)comp/

OBJ_DIR = obj/

EX_DIR  = ex/

UTIL_DIR  = util/

COMMON_OBJ =		 					\
				$(OBJ_DIR)list.o     \
				$(OBJ_DIR)heap.o     \
				$(OBJ_DIR)queue.o    \
				$(OBJ_DIR)map.o      \
				$(OBJ_DIR)mem.o      \
				$(OBJ_DIR)string.o   \
				$(OBJ_DIR)file.o     \
				$(OBJ_DIR)filepcx.o  \
				$(OBJ_DIR)filerem.o  \
				$(OBJ_DIR)filetxe.o  \
				$(OBJ_DIR)texture.o  \
				$(OBJ_DIR)draw.o     \
				$(OBJ_DIR)drbasic.o  \
				$(OBJ_DIR)flat4.o    \
				$(OBJ_DIR)flat.o     \
				$(OBJ_DIR)intlgt.o   \
				$(OBJ_DIR)portal.o   \
				$(OBJ_DIR)texflat.o  \
				$(OBJ_DIR)texintlg.o \
				$(OBJ_DIR)lighttab.o \
				$(OBJ_DIR)mesh.o     \
				$(OBJ_DIR)sector.o   \
				$(OBJ_DIR)object.o   \
				$(OBJ_DIR)scenery.o  \
 				$(OBJ_DIR)render.o   \
 				$(OBJ_DIR)rframe.o   \
 				$(OBJ_DIR)robjects.o \
 				$(OBJ_DIR)rsector.o  \
 				$(OBJ_DIR)rmesh.o    \
				$(OBJ_DIR)camsys.o   \
 				$(OBJ_DIR)engine.o   \
				$(OBJ_DIR)bpn.o      \
				$(OBJ_DIR)neuron.o   \
				$(OBJ_DIR)socket.o   \
				$(OBJ_DIR)svsocket.o \
 				$(OBJ_DIR)math.o		\
 				$(OBJ_DIR)refsys.o	\
 				$(OBJ_DIR)polygon.o	\
				$(OBJ_DIR)eventtab.o \
 				$(OBJ_DIR)cokbcont.o	\
 				$(OBJ_DIR)codata.o	\
 				$(OBJ_DIR)cocamsys.o	\
 				$(OBJ_DIR)comesh.o	\
 				$(OBJ_DIR)colsomni.o	\
 				$(OBJ_DIR)cofuncha.o	\
 				$(OBJ_DIR)comove.o	\
 				$(OBJ_DIR)cosecset.o

COMMON_HEADERS =					   \
				$(SRC_DIR)list.h     \
				$(SRC_DIR)heap.h     \
				$(SRC_DIR)queue.h    \
				$(SRC_DIR)map.h      \
				$(SRC_DIR)mem.h      \
				$(SRC_DIR)array.h    \
				$(SRC_DIR)string.h   \
				$(SRC_DIR)file.h     \
				$(SRC_DIR)filepcx.h  \
				$(SRC_DIR)filerem.h  \
				$(SRC_DIR)filetxe.h  \
				$(SRC_DIR)texture.h  \
				$(SRC_DIR)draw/draw.h     \
				$(SRC_DIR)lighttab.h \
				$(SRC_DIR)mesh.h     \
				$(SRC_DIR)sector.h   \
				$(SRC_DIR)object.h   \
				$(SRC_DIR)scenery.h  \
				$(SRC_DIR)render/render.h   \
				$(SRC_DIR)camsys.h   \
 				$(SRC_DIR)engine.h   \
				$(SRC_DIR)bpn.h      \
				$(SRC_DIR)neuron.h   \
				$(SRC_DIR)socket.h   \
				$(SRC_DIR)svsocket.h \
 				$(SRC_DIR)math.h		\
 				$(SRC_DIR)refsys.h	\
 				$(SRC_DIR)polygon.h	\
 				$(SRC_DIR)eventtab.h	\
 				$(COM_DIR)cokbcont.h	\
 				$(COM_DIR)codata.h	\
 				$(COM_DIR)cocamsys.h	\
 				$(COM_DIR)comesh.h	\
 				$(COM_DIR)colsomni.h	\
 				$(COM_DIR)cofuncha.h	\
 				$(COM_DIR)comove.h	\
 				$(COM_DIR)cosecset.h



# show init info
instruction: 			 	
				@echo Making:
				@echo make all        - compile library, examples, utils
				@echo make library    - compile 3DFC library object files
				@echo make examples   - compile examples, need library object files
				@echo make utils      - compile utils, need library object files
				@echo To compile a separate object file, example or util
				@echo pass its label in command line.
				@echo Ex.: make mem		  - compile the CMem library class
				@echo      make exmem 	  - compile the CMem test example
				@echo      make utbb		  - compile the Box Build util
				@echo See readme.txt to get more info.



# build all code available
all: 		   library examples utils
				@echo .
				@echo .
				@echo Library object files compiled and moved to ./OBJ.
				@echo Examples compiled and moved to ./EX, for more info see doc/examples.txt
				@echo Utils compiled and moved to ./UTIL/BIN, for more info see doc/util.txt
				@echo For more compilation instructions, just type: make
				@echo 3D Foundation Classes sources compiled. All ready to use.



# build only the library object files
commonlibrary:	   list mem string heap queue file filerem \
		  				render rframe robjects rsector rmesh    \
			        filepcx filetxe map scenery texture object \
			        sector mesh camsys bpn neuron engine \
				draw drbasic flat4 flat intlgt  \
				portal texflat texintlg \
				lighttab math polygon refsys cokbcont codata cocamsys \
				comesh cosecset cofuncha comove colsomni eventtab \
		  	   socket svsocket
				@echo .
				@echo .
				@echo Object files moved to ./OBJ.



# build only test examples
commontextexamples:	 exlist exmem exarray exstring \
 			 exheap exqueue exfile exfilerem extesttxe exmap \
			 exbpnand exbpnor exbpnxor exbpn exserver exclient \
			 endcommonexamples



# build only the graphics examples
commongraphicsexamples:	\
				exshowpcx \
				exscale exdraw \
				exdrawclip exfly \
				exlight exdrawtex exdrawgou exdrawgt \
				endcommonexamples



endcommonexamples:
				@echo .
				@echo .
				@echo Examples compiled and moved to ./EX.
				@echo For more compilation instructions, just type: make


# build only the utilitys
commonutils:		utbb
				@echo .
				@echo .
				@echo Utils compiled and moved to ./UTILS/BIN.
				@echo For more compilation instructions, just type: make




# THE LIBRARY SOURCES
###########################################################



mem:			$(SRC_DIR)mem.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)mem.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



eventtab:	$(SRC_DIR)eventtab.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)eventtab.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



math:			$(SRC_DIR)math.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)math.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



cokbcont:	$(COM_DIR)cokbcont.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)cokbcont.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



codata:		$(COM_DIR)codata.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)codata.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



colsomni:		$(COM_DIR)colsomni.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)colsomni.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



comesh:		$(COM_DIR)comesh.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)comesh.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



comove:		$(COM_DIR)comove.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)comove.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



cofuncha:		$(COM_DIR)cofuncha.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)cofuncha.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



cocamsys:	$(COM_DIR)cocamsys.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)cocamsys.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



cosecset:	$(COM_DIR)cosecset.cc
			   $(COMPILER) $(OPTIONS) -c $(COM_DIR)cosecset.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



polygon:		$(SRC_DIR)polygon.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)polygon.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



refsys:		$(SRC_DIR)refsys.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)refsys.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



object:		$(SRC_DIR)object.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)object.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



sector:		$(SRC_DIR)sector.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)sector.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



mesh:	 		$(SRC_DIR)mesh.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)mesh.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



camsys:	 	$(SRC_DIR)camsys.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)camsys.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



engine:	 	$(SRC_DIR)engine.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)engine.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



render:	 	$(SRC_DIR)render/render.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)render/render.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



rframe:	 	$(SRC_DIR)render/rframe.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)render/rframe.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



rsector:	 	$(SRC_DIR)render/rsector.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)render/rsector.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



robjects:	 	$(SRC_DIR)render/robjects.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)render/robjects.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



rmesh:	 	$(SRC_DIR)render/rmesh.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)render/rmesh.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



draw:			$(SRC_DIR)draw/draw.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw/draw.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



drbasic:		$(SRC_DIR)draw/drbasic.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw/drbasic.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



flat4:			$(SRC_DIR)draw/flat4.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw/flat4.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



flat:			$(SRC_DIR)draw/flat.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw/flat.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



intlgt:		$(SRC_DIR)draw/intlgt.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw/intlgt.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



portal:			$(SRC_DIR)draw/portal.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw/portal.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



texflat:		$(SRC_DIR)draw/texflat.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw/texflat.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



texintlg:		$(SRC_DIR)draw/texintlg.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)draw/texintlg.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



lighttab:	$(SRC_DIR)lighttab.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)lighttab.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



texture:		$(SRC_DIR)texture.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)texture.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



filepcx:		$(SRC_DIR)filepcx.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)filepcx.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



filerem:		$(SRC_DIR)filerem.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)filerem.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



filetxe:		$(SRC_DIR)filetxe.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)filetxe.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



scenery:		$(SRC_DIR)scenery.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)scenery.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



file:			$(SRC_DIR)file.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)file.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



queue:		$(SRC_DIR)queue.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)queue.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



heap:			$(SRC_DIR)heap.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)heap.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



string:		$(SRC_DIR)string.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)string.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



list:     	$(SRC_DIR)list.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)list.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



map:     	$(SRC_DIR)map.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)map.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



neuron:    	$(SRC_DIR)neuron.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)neuron.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



bpn: 	   	$(SRC_DIR)bpn.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)bpn.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



socket:   	$(SRC_DIR)socket.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)socket.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



svsocket:  	$(SRC_DIR)svsocket.cc
			   $(COMPILER) $(OPTIONS) -c $(SRC_DIR)svsocket.cc
				$(CMD_COPY_OBJ)
				$(CMD_DEL_OBJ)



# THE TEXT EXAMPLES
###########################################################



exarray:		$(EX_DIR)array.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)array.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) array.o -o array.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exmem:		$(EX_DIR)mem.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)mem.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) mem.o -o mem.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exlist:		$(EX_DIR)list.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)list.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) list.o -o list.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exstring:	$(EX_DIR)string.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)string.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) string.o -o string.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exheap:		$(EX_DIR)heap.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)heap.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) heap.o -o heap.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exqueue:		$(EX_DIR)queue.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)queue.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) queue.o -o queue.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exfile:		$(EX_DIR)file.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)file.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) file.o -o file.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exfilerem:	$(EX_DIR)filerem.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)filerem.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) filerem.o -o filerem.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exmap:		$(EX_DIR)map.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)map.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) map.o -o map.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



extesttxe:	$(EX_DIR)testtxe.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)testtxe.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) testtxe.o -o testtxe.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exbpn:		$(EX_DIR)bpn.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)bpn.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) bpn.o -o bpn.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exbpnxor:	$(EX_DIR)bpnxor.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)bpnxor.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) bpnxor.o -o bpnxor.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exbpnor:		$(EX_DIR)bpnor.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)bpnor.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) bpnor.o -o bpnor.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exbpnand:	$(EX_DIR)bpnand.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)bpnand.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) bpnand.o -o bpnand.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exserver:	$(EX_DIR)server.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)server.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) server.o -o server.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exclient:	$(EX_DIR)client.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)client.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) client.o -o client.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



# THE GRAPHICS EXAMPLES
###########################################################

						  

exfly:		$(EX_DIR)fly.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)fly.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) fly.o -o fly.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exdrawgt:	$(EX_DIR)drawgt.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)drawgt.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) drawgt.o -o drawgt.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

						  

exdrawgou:	$(EX_DIR)drawgou.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)drawgou.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) drawgou.o -o drawgou.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

						  

exdrawtex:	$(EX_DIR)drawtex.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)drawtex.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) drawtex.o -o drawtex.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exlight:		$(EX_DIR)light.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)light.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) light.o -o light.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exdraw:		$(EX_DIR)draw.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)draw.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) draw.o -o draw.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exscale:		$(EX_DIR)scale.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)scale.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) scale.o -o scale.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exshowpcx:	$(EX_DIR)showpcx.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)showpcx.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) showpcx.o -o showpcx.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



exdrawclip:	$(EX_DIR)drawclip.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(EX_DIR)drawclip.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) drawclip.o -o drawclip.exe $(EXTRA_LIBS)
				$(CMD_COPY_EXE)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)



# THE UTILS
###########################################################



utbb:			$(UTIL_DIR)bb/bb.cc $(COMMON_HEADERS) $(HEADERS)
			   $(COMPILER) $(OPTIONS) -c $(UTIL_DIR)bb/bb.cc
				$(COMPILER) $(OPTIONS) $(COMMON_OBJ) $(OBJ) bb.o -o bb.exe $(EXTRA_LIBS)
				$(CMD_COPY_UTIL)
				$(CMD_DEL_OBJ)
				$(CMD_DEL_EXE)

