/* Move object.

TIPS:
- this is one possible object component.
- this class derive from CNodeComponent, see nodecomp.h for more info.
*/



#include <string.h>

#include "../defs.h"

#include "../mem.h"

#include "../array.h"

#include "../string.h"

#include "../node.h"

#include "../list.h"

#include "../polygon.h"

#include "../nodepol.h"

#include "../nodeint.h"

#include "../map.h"

#include "../math.h"

#include "../mesh.h"

#include "../sector.h"

#include "../nodesec.h"

#include "../refsys.h"

#include "../camsys.h"

#include "../nodesec.h"

#include "../lighttab.h"

#include "../eventtab.h"

#include "../scenery.h"

#include "../object.h"

#include "../nodeobj.h"

#include "nodecomp.h"

#include "cocamsys.h"

#include "comesh.h"

#include "comove.h"



/* constructor - set the type variable from CNodeComponent
**********************************************************/
CComponentMove::CComponentMove ()
{
   m_dwMyTypeID = TYPE_MOVE;

   m_stDirection.fX = 0;
   m_stDirection.fY = 0;
   m_stDirection.fZ = 0;
   m_stAcceleration.fX = 0;
   m_stAcceleration.fY = 0;
   m_stAcceleration.fZ = 0;
   m_yAX = 0;
   m_yAY = 0;
   m_yAZ = 0;

   m_adwCaseSolidCollide [ 0 ] = SLOT_UNKNOWN;
   m_adwCaseSolidCollide [ 1 ] = 0;
   m_adwCaseSolidCollide [ 2 ] = 0;
   m_adwCasePortalCross [ 0 ] = SLOT_UNKNOWN;
   m_adwCasePortalCross [ 1 ] = 0;
   m_adwCasePortalCross [ 2 ] = 0;
}



/* Move the object between sectors.
**********************************************************/
void CComponentMove::mChangeSector ( CNodeSector* poDestSector )
{
   // remove the object from the sector
   CNode
      *poMyNode = m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRetValue ( m_poMyObj->m_dwMyID );
   m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRemoveKey ( m_poMyObj->m_dwMyID, FALSE );

   // add references to new sector
   poDestSector->m_oSector.m_oObjects.mAddKey ( m_poMyObj->m_dwMyID, (CNode*) poMyNode );

   // change sector
   m_poMyObj->m_poMySec = poDestSector;
}



/* Try make object move
return value 1 or one value returned in slot activation
**********************************************************/
PTRVAL CComponentMove::mDoMove ()
{
   // get object reference system
   CComponentCameraSystem
      *poComp = (CComponentCameraSystem*) m_poMyObj->mRetComponent ( SLOT_REFSYS );
   if ( poComp == NULL )
      ABORT ( MSG_ERROR_NO_TYPE, __FILE__, __LINE__ );
   CCameraSystem
      *poRefSys = &poComp->m_oCamSys;

   // update direction vector with acceleration vector
   m_stDirection.fX += m_stAcceleration.fX;
   m_stDirection.fY += m_stAcceleration.fY;
   m_stDirection.fZ += m_stAcceleration.fZ;
   
   // update base - make rotation
   poRefSys->mRotateTransBase ( m_yAX, m_yAY, m_yAZ );

   // define possible new position
   tagfXYZ
      stPosNew;
   stPosNew.fX = poRefSys->m_stPos.fX + m_stDirection.fX;
   stPosNew.fY = poRefSys->m_stPos.fY + m_stDirection.fY;
   stPosNew.fZ = poRefSys->m_stPos.fZ + m_stDirection.fZ;
      
   // detect collision between sector where object is inside and object
   BYTE
      yCode;
   CNodePolygon
      *poPol = m_poMyObj->m_poMySec->m_oSector.mDetectCollision ( stPosNew, poRefSys->m_fRadius, yCode );

   // return value
   PTRVAL
      pRetVal = 1;

   // if not colliding with parent sector, test between objects inside sector
   if ( poPol == NULL ) {

      // number of objects inside sector
      DWORD
         dwNum = m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRetSize ();

      // for each object
      for ( DWORD dwcInd = 0; dwcInd < dwNum; dwcInd ++ ) {

         // get key (object ID)
         DWORD
            dwKey = m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRetKey ( dwcInd );

         // get object pointer
         CNodeObject
            *poObj = (CNodeObject*) m_poMyObj->m_poMySec->m_oSector.m_oObjects.mRetValue ( dwKey );

         CNodeComponent
            *poBody = (CNodeComponent*) poObj->m_oObject.mRetComponent ( SLOT_BODY );

         // verify if has a body, if don't - skip object
         if ( poBody == NULL )
            continue;
       
         // verify if body not found, or not compatible type, or
         // is the same object (impossible test collide with it self)
         if ( &poObj->m_oObject == m_poMyObj ||
              poBody == NULL ||
              poBody->m_dwMyTypeID != TYPE_MESH )
            continue;
         else
            // make test
            poPol = ( (CComponentMesh*) poBody )->m_oMesh.mDetectCollision ( stPosNew, poRefSys->m_fRadius, yCode );

         // if collide
         if ( poPol != NULL )
            break;
      }
   }

   // if have collided
   if ( poPol != NULL ) {

      // test cases

      // if portal crossing
      if ( yCode & 2 ) {

         // verify if need activate other slot
         if ( m_adwCasePortalCross [ 0 ] != SLOT_UNKNOWN )
         
            // activate slot, return it's value
            pRetVal = m_poMyObj->mActivateComponent ( m_adwCasePortalCross [ 0 ],
                                                      m_adwCasePortalCross [ 1 ],
                                                      m_adwCasePortalCross [ 2 ],
                                                      (PTRVAL) poPol );

         // if returned value is not 0, do default action                                             
         if ( pRetVal )

            // move the instance to the destin sector
            mChangeSector ( poPol->m_oPolygon.m_stPortal.poSector );
      }   

      // if collide ONLY with solid poly (NO portal cross)
      if ( yCode == 1 ) {

         // verify if need activate other slot
         if ( m_adwCaseSolidCollide [ 0 ] != SLOT_UNKNOWN )
         
            // activate slot, return it's value
            pRetVal = m_poMyObj->mActivateComponent ( m_adwCaseSolidCollide [ 0 ],
                                                      m_adwCaseSolidCollide [ 1 ],
                                                      m_adwCaseSolidCollide [ 2 ],
                                                      (PTRVAL) poPol );

         // if returned value is not 0, do default action                                             
         if ( pRetVal ) {

            // go back
            m_stDirection.fX -= poPol->m_oPolygon.m_stNormal.fX * poRefSys->m_fSpeed;
            m_stDirection.fY -= poPol->m_oPolygon.m_stNormal.fY * poRefSys->m_fSpeed;
            m_stDirection.fZ -= poPol->m_oPolygon.m_stNormal.fZ * poRefSys->m_fSpeed;
         }   
      }   
      
   }

   // change position
   if ( pRetVal ) { 

      poRefSys->m_stPos.fX += m_stDirection.fX;
      poRefSys->m_stPos.fY += m_stDirection.fY;
      poRefSys->m_stPos.fZ += m_stDirection.fZ;
   }  
   
   return pRetVal;
}
      


/* Execute one action
pData1 - 0 - set this component data with source component
         1 - set what to do if collide with solid poly or cross portal
         2 - update object position/orietation and affset vectors 
         3 - set direction vector
         4 - set acceleration vector
         5 - offset direction vector
         6 - offset acceleration vector
         7 - set rotation angles
pData2 - if pData1 is 0 - point the source CComponentMove
         if pData1 is 1 - point an array of DWORD having the actions of what do case
                          solid poly collion is detected (see array format in comove.h).
                          If NULL, no array setup is done. 
         if pData1 between 2 and 6- pointer to struct tagfXYZ with needed data
pData3 - if pData1 is 1 - point an array of DWORD having the actions of what do case
                          portal crossing is detected (see array format in comove.h).
                          If NULL, no array setup is done. 
         other pData1 values - not used
Return - if pData1 is 2 - value 1;
                          or - case other slot is activated - the returned value;
                          if returned value in activation is 0, no position is
                          updated after slot activation; else position is updated with
                          m_stDirection value.
         other pData1 values - return 0
**********************************************************/
PTRVAL CComponentMove::mActivate ( PTRVAL pData1, PTRVAL pData2, PTRVAL pData3 )
{
   // the given vector - if exist
   tagfXYZ
      *pstVector = NULL;
   if ( pData1 >= 3 )
      pstVector = (tagfXYZ*) pData2;
             
   // execute one specific action
   switch ( pData1 ) {
   
      case 0:          // copy
         {
            CComponentMove
               *poSrc = (CComponentMove*) pData2;  // source component

            m_adwCaseSolidCollide [ 0 ] = poSrc->m_adwCaseSolidCollide [ 0 ];
            m_adwCaseSolidCollide [ 1 ] = poSrc->m_adwCaseSolidCollide [ 1 ];
            m_adwCaseSolidCollide [ 2 ] = poSrc->m_adwCaseSolidCollide [ 2 ];
            m_adwCasePortalCross [ 0 ] = poSrc->m_adwCasePortalCross [ 0 ];
            m_adwCasePortalCross [ 1 ] = poSrc->m_adwCasePortalCross [ 1 ];
            m_adwCasePortalCross [ 2 ] = poSrc->m_adwCasePortalCross [ 2 ];
            m_stDirection.fX = poSrc->m_stDirection.fX;
            m_stDirection.fY = poSrc->m_stDirection.fY;
            m_stDirection.fZ = poSrc->m_stDirection.fZ;
            m_stAcceleration.fX = poSrc->m_stAcceleration.fX;
            m_stAcceleration.fY = poSrc->m_stAcceleration.fY;
            m_stAcceleration.fZ = poSrc->m_stAcceleration.fZ;
            m_yAX = poSrc->m_yAX;
            m_yAY = poSrc->m_yAY;
            m_yAZ = poSrc->m_yAZ;
         }
         break;          
   
      case 1:          // setup action
         {
            // if collide with solid poly 
            DWORD
               *padwColl = (DWORD*) pData2;
            if ( padwColl != NULL ) {
               m_adwCaseSolidCollide [ 0 ] = padwColl [ 0 ];
               m_adwCaseSolidCollide [ 1 ] = padwColl [ 1 ];
               m_adwCaseSolidCollide [ 2 ] = padwColl [ 2 ];
            }   

            // if cross portal
            DWORD
               *padwCros = (DWORD*) pData3;
            if ( padwCros != NULL ) {
               m_adwCasePortalCross [ 0 ] = padwCros [ 0 ];
               m_adwCasePortalCross [ 1 ] = padwCros [ 1 ];
               m_adwCasePortalCross [ 2 ] = padwCros [ 2 ];
            }
         }
         break;          
   
      case 2:          // update object system
         return mDoMove ();
         break;

      case 3:           // set direction
         m_stDirection.fX = pstVector->fX;
         m_stDirection.fY = pstVector->fY;
         m_stDirection.fZ = pstVector->fZ;
         break;

      case 4:           // set acceleration
         m_stAcceleration.fX = pstVector->fX;
         m_stAcceleration.fY = pstVector->fY;
         m_stAcceleration.fZ = pstVector->fZ;
         break;

      case 5:           // offset direction
         m_stDirection.fX += pstVector->fX;
         m_stDirection.fY += pstVector->fY;
         m_stDirection.fZ += pstVector->fZ;
         break;

      case 6:           // offset acceleration
         m_stAcceleration.fX += pstVector->fX;
         m_stAcceleration.fY += pstVector->fY;
         m_stAcceleration.fZ += pstVector->fZ;
         break;

      case 7:           // set rotation Nangs
         {
            BYTE
               *payRot = (BYTE*) pData2;

            m_yAX = payRot [ 0 ];
            m_yAY = payRot [ 1 ];
            m_yAZ = payRot [ 2 ];
         }
         break;   
   }

   return 0;
}
