/* Set portal edges and bounding box from a given poly.
Use the edge scanner for file flat.cc. */



#include <string.h>
#include <math.h>

#include "../defs.h"

#include "../mem.h"

#include "../string.h"

#include "../texture.h"

#include "../lighttab.h"

#include "../asm/draw.inc"

#include "draw.h"



/* Reset the given edge buffer (pastEdgeBuffer): set the
start limit to the minimum x and the end limit to maximum
x acording the frame buffer. Do not perform clipping.
**********************************************************/
void CDraw::mResetClipEdgeBuffer ( tagInterval* pastEdgeBuffer )
{
   tagInterval
      *pstInt = pastEdgeBuffer;

   // scan all edge buffer setting their values
   for ( INT ncYScanLine = 0; ncYScanLine < m_nResY; ncYScanLine ++ ) {
      pstInt->nStart = 0;
      pstInt->nEnd = m_nMaxX;
      pstInt ++;
   }
}



/* Set the given clip buffer using the edges defined by the
given polygon.
The given polygon is clipped through the given edge buffer.
pastXYZes have the array of polygon vertexes in clockwise
or anticlockwise order used to define the edge buffer.
nNumVertexes is the number of polygon vertexes.
pastEdgeBuffer is the clip edge buffer, must have the same size
stClipBox is the bounding box around the clip polygon that can be
used in fast poly clip, enter the current clip box and return the new.
as the vertical resolution (Y).
return TRUE - poly drawn (not totaly clipped) / FALSE - poly
totaly invisible (completely clipped)
**********************************************************/
BOOL CDraw::mSetClipEdgeBuffer ( tagXY* pastXYZes, INT nNumVertexes,
                                 tagInterval* pastEdgeBuffer,
                                 tagRectangle& stClipBox )
{
   // flag if poly has some part (or completely) visible
   BOOL
      bVisible = FALSE;        // originaly not visible
   
   // verify if have enough number of vertexes
   if ( nNumVertexes < 3 )
      ABORT ( MSG_ERROR_LOW_VERTEXES, __FILE__, __LINE__ )

   // seach top and bottom vertexes
   INT
      nYMin = pastXYZes [ 0 ].nY,   // y of the top vertex
      nYMax = pastXYZes [ 0 ].nY,   // y of the bottom vertex
      nPosMin = 0,       // the index of the top vertex (upper in screen)
      nPosMax = 0;       // the index of the bottom vertex (lower in screen)
   // verify vertexes
   for ( INT ncVert = 1; ncVert < nNumVertexes; ncVert ++ ) {

      // verify if array vertex is greater than the current bottom vertex
      if ( pastXYZes [ ncVert ].nY > nYMax ) {
         nYMax = pastXYZes [ ncVert ].nY;
         nPosMax = ncVert;
      }

      // verify if array vertex is less than the current top vertex
      if ( pastXYZes [ ncVert ].nY < nYMin ) {
         nYMin = pastXYZes [ ncVert ].nY;
         nPosMin = ncVert;
      }
   }

   // verify if top and bottom are the same, if yes abort edge fill
   if ( nYMax == nYMin ) {
      stClipBox.nStartY = nYMax;
      stClipBox.nEndY = nYMax;
      return bVisible;
   }

   // scan forward in array of vertexes starting on top and going down
   // With this you scan one side of the polygon.
   INT
      nPos1 = nPosMin,         // the first vertex to be tested
      nPos2 = nPos1;           // the next vertex (used in same test)
   while ( nPos1 != nPosMax ) {

      // define the next vertex position in vertex array
      nPos2 ++;
      // if no more vertexes go to array start
      if ( nPos2 == nNumVertexes )
         nPos2 = 0;

      // define polygon edges, the X for each scanline defined by the line
      // vertex 1 ----> vertex 2
      mScanFlatEdge ( m_pastForwardEdges, pastXYZes [ nPos1 ], pastXYZes [ nPos2 ] );

      // define current vertex as the next vertex
      nPos1 = nPos2;
   }

   // scan backward in array of vertexes starting on top and going down
   // With this you scan one side of the polygon.
   nPos1 = nPosMin;         // the first vertex to be tested
   nPos2 = nPos1;           // the next vertex (used in same test)
   while ( nPos1 != nPosMax ) {

      // define the previous vertex position in vertex array
      nPos2 --;
      // if no more vertexes go to array end
      if ( nPos2 == -1 )
         nPos2 = nNumVertexes - 1;

      // define polygon edges, the X for each scanline defined by the line
      // vertex 1 ----> vertex 2
      mScanFlatEdge ( m_pastBackwardEdges, pastXYZes [ nPos1 ], pastXYZes [ nPos2 ] );

      // define current vertex as the previous vertex
      nPos1 = nPos2;
   }

   // Update edge buffer using 3 loops: the first set the area
   // between clip buffer start and clipping polygon begin. The second
   // set the area over the clipping polygon. The third set the area
   // between end clipping polygon and end buffer.
   // define start buffer position (Y) changed by clipping polygon
   INT
      nY = nYMin;
   if ( nY < 0 )
      nY = 0;     // start buffer line changed by polygon edges
   // define bottom buffer position (Y) changed by clipping polygon
   INT
      nYBottom = nYMax;
   if ( nYBottom > m_nMaxY )
      nYBottom = m_nMaxY;
   // start edge buffer position
   tagInterval
      *pstInt = pastEdgeBuffer,
      *pstEnd = &pastEdgeBuffer [ m_nMaxY ];

   // verify if can change the Y up clip box limit
   if ( nY > stClipBox.nStartY )
      stClipBox.nStartY = nY;
   // verify if can change the Y bottom clip box limit
   if ( nYBottom < stClipBox.nEndY )
      stClipBox.nEndY = nYBottom;

   // store the less and greatter X for this clip edge buffer
   tagInterval
      stXLim;
   // REVIEW
   //stXLim.nStart = m_nMaxX;
   //stXLim.nEnd = 0;

   // scan edge buffer begin to start polygon edges
   if ( nY < m_nResY )
      for ( INT ncY = 0; ncY < nY; ncY ++ ) {
         // set edge buffer: no space available for drawing
         if ( pstInt <= pstEnd ) {
            pstInt->nStart = m_nMaxX;
            pstInt->nEnd = 0;
            if ( pstInt < pstEnd ) 
               pstInt ++;
         }
      }

   // scan edge buffer from top polygon to bottom polygon
   // if X values for this scanline are swapped
   if ( m_pastBackwardEdges [ nY ].nX < m_pastForwardEdges [ nY ].nX )
      while ( nY <= nYBottom ) {

         if ( nY < m_nResY ) {

            // make edge buffer X clip and define new buffer X limits
            // clip start X
            if ( m_pastBackwardEdges [ nY ].nX > pstInt->nStart ) {
               if ( pstInt <= pstEnd )
                  if ( m_pastBackwardEdges [ nY ].nX < 0 )
                     pstInt->nStart = 0;
                  else
                     pstInt->nStart = m_pastBackwardEdges [ nY ].nX;

               // search the minimum X in clip edge buffer
               if ( pstInt->nStart < stXLim.nStart )
                  stXLim.nStart = pstInt->nStart;
            }

            // clip end X
            if ( m_pastForwardEdges [ nY ].nX < pstInt->nEnd ) {
               if ( pstInt <= pstEnd ) 
                  if ( m_pastForwardEdges [ nY ].nX > m_nMaxX )
                     pstInt->nEnd = m_nMaxX;
                  else
                     pstInt->nEnd = m_pastForwardEdges [ nY ].nX;
   
               // search the maximum X in clip edge buffer
               if ( pstInt->nEnd > stXLim.nEnd )
                  stXLim.nEnd = pstInt->nEnd;
            }

            // if line visible
            if ( pstInt->nStart >= stClipBox.nStartX && pstInt->nEnd <= stClipBox.nEndX )
               bVisible = TRUE;   // poly is visible (partialy or completely)

            if ( pstInt < pstEnd ) 
               pstInt ++;
         }
         
         // goto next scanline
         nY ++;
      }
   else 
      while ( nY <= nYBottom ) {

         if ( nY < m_nResY ) {

            // make edge buffer X clip and define new buffer X limits
            // clip start X
            if ( m_pastForwardEdges [ nY ].nX > pstInt->nStart ) {
               if ( pstInt <= pstEnd ) 
                  if ( m_pastForwardEdges [ nY ].nX < 0 )
                     pstInt->nStart = 0;
                  else
                     pstInt->nStart = m_pastForwardEdges [ nY ].nX;

               // search the minimum X in clip edge buffer
               if ( pstInt->nStart < stXLim.nStart )
                  stXLim.nStart = pstInt->nStart;
            }

            // clip end X
            if ( m_pastBackwardEdges [ nY ].nX < pstInt->nEnd ) {
               if ( pstInt <= pstEnd )
                  if ( m_pastBackwardEdges [ nY ].nX > m_nMaxX )
                     pstInt->nEnd = m_nMaxX;
                  else
                     pstInt->nEnd = m_pastBackwardEdges [ nY ].nX;

               // search the maximum X in clip edge buffer
               if ( pstInt->nEnd > stXLim.nEnd )
                  stXLim.nEnd = pstInt->nEnd;
            }

            // if line visible
            if ( pstInt->nStart >= stClipBox.nStartX && pstInt->nEnd <= stClipBox.nEndX )
               bVisible = TRUE;   // poly is visible (partialy or completely)

            if ( pstInt < pstEnd ) 
               pstInt ++;
         }
         
         // goto next scanline
         nY ++;
      }

   // scan end clip polygon to buffer end
   for ( INT ncY = nYBottom + 1; ncY < m_nResY; ncY ++ ) {
      if ( nY < m_nResY ) {
         // set edge buffer: no space available for drawing
         if ( pstInt <= pstEnd ) {
            pstInt->nStart = m_nMaxX;
            pstInt->nEnd = 0;
         }
         if ( pstInt < pstEnd ) 
            pstInt ++;
      }
   }

   // REVIEW
   // verify if have updated X start and end, if no use original values
   //if ( bVisible ) {  // poly visible, must have good values for X limits
      //if ( stXLim.nStart == stClipBox.nEndX )
         //stXLim.nStart = stClipBox.nStartX;
      //if ( stXLim.nEnd == stClipBox.nStartX )
         //stXLim.nEnd = stClipBox.nEndX;
   //}

   // verify if can update clip box X limits
   /*if ( stClipBox.nStartX < stXLim.nStart )
      stClipBox.nStartX = stXLim.nStart;
   if ( stClipBox.nEndX > stXLim.nEnd )
      stClipBox.nEndX = stXLim.nEnd;*/

/*for (int n=0;n<320;n++){
   if(pastEdgeBuffer[n].nStart<0)
      ABORT ( "baixo", __FILE__, __LINE__ )
   if(pastEdgeBuffer[n].nEnd>320)
      ABORT ( "alto", __FILE__, __LINE__ )
}  */

   return bVisible;
}

